function [SA_ntp, CT_ntp, p_ntp, p_bar] = gsw_ntp_bottle_to_cast(SA_bottle,CT_bottle,p_bottle,SA,CT,p,drho)

% gsw_ntp_bottle_to_cast        Absolute Salinity, Conservative Temperature
%                                 and pressure on a neighbouring cast using
%                             the neutral tangent plane  (75-term equation)
%==========================================================================
%
% USAGE:
%  [SA_ntp, CT_ntp, p_ntp, p_bar] = ...
%       gsw_ntp_bottle_to_cast(SA_bottle,CT_bottle,p_bottle,SA,CT,p,{drho})
%
% DESCRIPTION:
%  This function finds the Absolute Salinity, Conservative Temperature and 
%  pressure on the casts, which have the same specific volume referred to 
%  the mid-point pressure, p_bar, as the bottle.  
%
%  This programme assumes that the neighbouring cast is stable.  It 
%  searches in one direction only, based on the initial value of the 
%  difference in the locally referenced specific volume.  It searches for
%  the corresponding locally referenced specific volume on the neighbouring 
%  cast starting from the bottle pressure.  
%  
%  Note that this function is intended to work on stabile profiles.  If 
%  there are instabillities then there are mupliple solutions it will
%  generally not find all the crossings, the results may including stable 
%  unstable zero crossings.
%
%--------------------------------------------------------------------------
%                    Still working on this section
%  if drho is included it does a height adjustment of the surface.
%--------------------------------------------------------------------------
%
% INPUT:  
%  SA_bottle =  Absolute Salinity of the bottle                    [ g/kg ]
%  CT_bottle =  Conservative Temperature of the bottle(ITS-90)    [ deg C ]
%  p_bottle  =  sea pressure of the bottle                         [ dbar ]
%         ( i.e. absolute pressure - 10.1325 dbar )
%  SA  =  Absolute Salinity of the neighbouring cast               [ g/kg ]
%  CT  =  Conservative Temperature of the neighbouring cast       [ deg C ]
%  p   =  sea pressure of the neighbouring cast                    [ dbar ]
%         ( i.e. absolute pressure - 10.1325 dbar )
%
% OPTIONAL:
%  drho
%
% OUTPUT:
%  SA_ntp = Absolute Salinity of the neutral tangent plane         [ g/kg ]
%  CT_ntp = Conservative Temperature of the neutral tangent plane [ deg C ]
%  p_ntp  = pressure of the neutral tangent plane                  [ dbar ]
%  p_bar  = the average pressure of the neutral tangent plane      [ dbar ]
%
%  SA_bottle, CT_bottle and p_bottle need to be single values, having 
%  dimensions 1x1.
%  SA, CT and p need to have the same dimensions Mx1 or MxN, where M is the
%  number of bottles in the cast and N is the number of profiles.  Note 
%  that they need to be arranged in a column, if they are inputed as a row
%  the programme will assume they are seperate casts. 
%
% AUTHOR: David Jackett
%  Modified by Guillaume Serazin, Stefan Riha and Paul Barker
%
% VERSION NUMBER: 3.05.5 (1st September, 2016)
%
% REFERENCES: 
%  Jackett, D.R., and T.J. McDougall, 1997: A Neutral Density Variable for 
%   the World's Oceans. J. Phys. Oceanogr., 27, pp. 237263.
%
%  The software is available from http://www.TEOS-10.org
%
%==========================================================================

delta = 1e-12; % The accepted tolerance when calculating the locally 
% referenced specific volume difference between the bottle and the 
% neighbouring cast.  Note that this is approximately equal to 0.1 dbar.

[profile_length, Number_of_profiles] = size(SA);
size_dummy = size(SA);

if ~exist('drho','var')
    dspecvol = zeros(size_dummy); % dspecvol is a zero vector
else
    dspecvol = 1./drho;
    dspecvol(drho == 0) = 0;
end

Idata = 1:Number_of_profiles;

size_dummy(1) = 1;
SA_ntp = NaN(size_dummy);
CT_ntp = SA_ntp;
p_ntp = SA_ntp;
p_bar = SA_ntp;

% discard land, Idata may contain the indices of remaining points
[SA_bottle, CT_bottle, p_bottle, SA, CT, p, dspecvol, Idata] = gsw_discard_land(SA_bottle, CT_bottle, p_bottle, SA, CT, p, dspecvol, Idata);

[profile_length, Number_of_profiles] = size(SA);

Iwet = [1:Number_of_profiles];

p0_stacked = repmat(p_bottle(:)',[profile_length 1]);

I_bottle_above = sum(p0_stacked >= p,1);                         % index of adjacent bottle looking up, values 1 to nz.
bottles_above = p_bottle(:)' < p(1,:);                           % bottle above shallowest cast Data Point
I_bottle_above(bottles_above) = 1;                               % start at the uppermost cast pair
I3d = I_bottle_above + profile_length*(0:Number_of_profiles-1);  % changing to 3-d index, based on casts supplied.
Iinc = NaN(1,Number_of_profiles);                                % increment of the 3-d index

SA_tmp = SA(I3d);
CT_tmp = CT(I3d);
p_tmp = p(I3d);

searched = ~isnan(SA_tmp);

Idata_nn = ~isnan(SA); 
Idata_intergral = cumsum(Idata_nn,1);
su = ones(size(Idata_intergral));
su(Idata_intergral~=0) = 0;
Ishallowest = sum(su,1) + 1;                    % vertical index of shallowest bottle
if any(Ishallowest < 1 | Ishallowest > profile_length)
    error('gsw_ntp_bottle_to_cast: error # 1')          % land should have been discarded above
end

SA_dummy = SA(end:-1:1,:,:);
Idata_nn = ~isnan(SA_dummy);
Idata_intergral = cumsum(Idata_nn,1);
su = ones(size(Idata_intergral));
su(Idata_intergral~=0) = 0;
%su = zeros(size(Idata_intergral));
Ideepest = sum(su,1) + 1;
Ideepest = profile_length - Ideepest + 1;        % vertical index of deepest data point
if any(Ideepest < 1 | Ideepest > profile_length)
    error('gsw_ntp_bottle_to_cast: error # 2')           % land should have been discarded above
end

go_shallow_old = false(1,Number_of_profiles);
go_deep_old = false(1,Number_of_profiles);

bisect = false(1,Number_of_profiles);
done = false(1,Number_of_profiles);
Number_of_iterations = 0;
[vp0,vp1,vp2,vp3,vp4] = gsw_specvol_p_parts(SA_bottle,CT_bottle);

while any(~done)
    
    Number_of_iterations = Number_of_iterations + 1;
    
    p_mid = 0.5*(p_bottle + p_tmp);
    z_mid = p_mid.*1e-4;
    v_part_bottle = vp0 + z_mid.*(vp1 + z_mid.*(vp2 + z_mid.*(vp3 + z_mid.*vp4)));

    [vp0_cast,vp1_cast,vp2_cast,vp3_cast,vp4_cast] = gsw_specvol_p_parts(SA_tmp,CT_tmp);
    v_part_cast = vp0_cast + z_mid.*(vp1_cast + z_mid.*(vp2_cast + z_mid.*(vp3_cast + z_mid.*vp4_cast)));
    
    v_local_diff =  v_part_bottle - v_part_cast + dspecvol;

    go_shallow = (v_local_diff >=  delta);   % The intercept is shallower
    go_deep = (v_local_diff <= -delta);      % The intercept is deeper 
        
    searched(~isnan(v_local_diff)) = true;
    
    % If v_local_diff, the locally referenced specific volume difference between
    % the bottle and the neighbouring cast, is NaN at the current pressure 
    % but it is well defined elsewhere in the watercolumn.
    nskip_up = I_bottle_above(Iwet) - Ideepest(Iwet);
    nskip_down = Ishallowest(Iwet) - I_bottle_above(Iwet);
    
    go_shallow_nan = isnan(v_local_diff)  & ~searched & (nskip_up>0);
    go_deep_nan = isnan(v_local_diff) & ~searched & (nskip_down>0);
    
    go_shallow = go_shallow | go_shallow_nan;
    go_deep = go_deep | go_deep_nan;
    
    final = abs(v_local_diff) < delta;  % The calculated salinty, temperature and 
                                        % pressure are within the tolerence level. 
    
    SA_ntp(Idata(Iwet(final))) = SA_tmp(final);
    CT_ntp(Idata(Iwet(final))) = CT_tmp(final);
    p_ntp(Idata(Iwet(final))) = p_tmp(final);
    p_bar(Idata(Iwet(final))) = p_mid(final);

    cfb = (go_shallow_old & go_deep & searched); % crossed from below
    cfa = (go_deep_old & go_shallow & searched); % crossed from above
    crossed = cfb | cfa;
    start_bis = (crossed & ~bisect);   % start bisection here
    bisect = (bisect | start_bis);     % bisect here
    
    search_initial = (go_deep | go_shallow) & ~bisect & ~final;
    
    Iinc(Iwet(go_deep & search_initial)) = 1;
    Iinc(Iwet(go_shallow & search_initial)) = -1;
    
    Iinc(Iwet(go_deep_nan & search_initial)) = nskip_down(Iwet(go_deep_nan & search_initial));
    Iinc(Iwet(go_shallow_nan & search_initial)) = -nskip_up(Iwet(go_shallow_nan & search_initial));
        
    I_bottle_above = I_bottle_above + Iinc;
    I3d = I3d + Iinc;
    Iwet_tmp = I_bottle_above(Iwet(search_initial));  
    I3d_wet = I3d(Iwet(search_initial)); 
    
    out = (Iwet_tmp < 1) | (Iwet_tmp > profile_length);   % above or below domain 
    out2 = false(1,length(final)); 
    out2(search_initial) = out; 
    
    done = (isnan(v_local_diff) & searched) | final | out2; 
    
    I3d_wet = I3d_wet(~out);
    search_initial = search_initial(~done);
    bisect = bisect(~done);
    crossed = crossed(~done);
    searched = searched(~done);
    
    if ~all((search_initial & ~bisect) | (~search_initial & bisect)) % either bisect or keep searching
        error('gsw_ntp_bottle_to_cast: error # 3, Something is wrong, there may be a NaN in the input data)')
    end
    
    if Number_of_iterations > 1
        SA1 = SA1(~done);
        CT1 = CT1(~done);
        p1 = p1(~done);
        
        SA2 = SA2(~done);
        CT2 = CT2(~done);
        p2 = p2(~done);
        
        SA2(crossed) = SA1(crossed);
        CT2(crossed) = CT1(crossed);
        p2(crossed) = p1(crossed);
    else
        SA2 = SA_tmp(~done);
        CT2 = CT_tmp(~done);
        p2 = p_tmp(~done);
    end
    
    SA1 = SA_tmp(~done);
    CT1 = CT_tmp(~done);
    p1 = p_tmp(~done);
    
    % data for next evaluation of the difference in the locally referenced 
    % specific volume between the bottle and the cast
    p_bottle = p_bottle(~done);
    dspecvol = dspecvol(~done);
    vp0 = vp0(~done);
    vp1 = vp1(~done);
    vp2 = vp2(~done);
    vp3 = vp3(~done);
    vp4 = vp4(~done);
    
    SA_tmp = NaN(1,sum(~done));
    CT_tmp = NaN(1,sum(~done));
    p_tmp = NaN(1,sum(~done));
    
    SA_tmp(search_initial) = SA(I3d_wet);
    CT_tmp(search_initial) = CT(I3d_wet);
    p_tmp(search_initial) = p(I3d_wet);
    
    if any(bisect)
        SA_tmp(bisect) = 0.5*(SA1(bisect) + SA2(bisect));
        CT_tmp(bisect) = 0.5*(CT1(bisect) + CT2(bisect));
        p_tmp(bisect) = 0.5*(p1(bisect) + p2(bisect));
    end
    
    go_shallow_old = go_shallow(~done);
    go_deep_old = go_deep(~done);
    
    Iwet = Iwet(~done);
end

end


function [SA_bottle,CT_bottle,p_bottle,SA,CT,p,dspecvol,Idata] = gsw_discard_land(SA_bottle,CT_bottle,p_bottle,SA,CT,p,dspecvol,Idata)

% gsw_discard_land                         eliminate land from the data set
%==========================================================================
%
% USAGE:
%  [SA_bottle,CT_bottle,p_bottle,SA,CT,p,dspecvol,Idata] = ...
%     gsw_discard_land(SA_bottle,CT_bottle,p_bottle,SA,CT,p,dspecvol,Idata)
%
% DESCRIPTION:
%  This function eliminates casts that do not have an initial estimate of 
%  the neutral surface. It is assumed that these locations are on land.
%
% INPUT:  
%  SA_bottle =  Absolute Salinity of the bottle                    [ g/kg ]
%  CT_bottle =  Conservative Temperature of the bottle(ITS-90)    [ deg C ]
%  p_bottle  =  sea pressure of the bottle                         [ dbar ]
%         ( i.e. absolute pressure - 10.1325 dbar )
%  SA  =  Absolute Salinity of the neighbouring cast               [ g/kg ]
%  CT  =  Conservative Temperature of the neighbouring cast       [ deg C ]
%  p   =  sea pressure of the neighbouring cast                    [ dbar ]
%         ( i.e. absolute pressure - 10.1325 dbar )
%  dspecvol
%  Idata
%
% OUTPUT:
%  output variables are the same as the input variables
%
% AUTHOR: David Jackett
%  Modified by Guillaume Serazin, Stefan Riha and Paul Barker
%
% VERSION NUMBER: 3.05.5 (1st September, 2016)
%
% REFERENCES: 
%  Jackett, D.R., and T.J. McDougall, 1997: A Neutral Density Variable for 
%   the World's Oceans. J. Phys. Oceanogr., 27, pp. 237263.
%
%  The software is available from http://www.TEOS-10.org
%
%==========================================================================

Iwet = ~isnan(SA_bottle);
Iwet = Iwet(:)';
Idry = all(isnan(SA));
Idry = Idry(:)';
Iwet = Iwet & ~Idry;

SA_bottle = SA_bottle(Iwet);
CT_bottle = CT_bottle(Iwet);
p_bottle = p_bottle(Iwet);

SA = SA(:,Iwet);
CT = CT(:,Iwet);
p = p(:,Iwet);

Idata = Idata(Iwet);

dspecvol = dspecvol(Iwet);

end
