function [SAns,CTns,pns] = gsw_ns2d(SA,CT,p,SAns,CTns,pns,e1t,e2t,settings)

% gsw_ns2d
%==========================================================================
% 
% USAGE: 
%  [SAns_i, CTns_i, pns_i] = gsw_ns2d(SA,CT,p,SAns,CTns,pns,e1t,e2t)
%
% DESCRIPTION:
%  Optimize density surfaces to minimise the fictitious diapycnal diffusivity
%
%  Optimize density surface using an iterative method to minimise
%  fictitious diapycnal diffusivity according 'A new method of
%  forming approximately neutral surfaces, Klocker et al., Ocean
%  Science, 5, 155-172, 2009.'
%
%
% INPUT:
%  SA  =  Absolute Salinity                                        [ g/kg ]
%  CT  =  Conservative Temperature (ITS-90)                       [ deg C ]
%  p   =  sea pressure                                             [ dbar ]
%         ( i.e. absolute pressure - 10.1325 dbar )
%           SAns        Absolute Salinity on initial density surface
%           CTns        Conservative Temperature on initial density
%                       surface
%           pns         pressure on initial density surface
%           e1t         grid length in meters in x-direCTion
%           e2t         grid length in meters in y-direCTion
%
%  settings    set in main window.
%            wrap        'none'
%                        'long'
%
% OUTPUT:
%  SAns =  Absolute Salinity on optimized surface
%  CTns =  Conservative Temperature on optimized surface
%  pns  =  pressure on optimized surface
%
% AUTHOR:
%  Andreas Klocker
% Modified:
%  Paul Barker and Trevor McDougall                    [ help@teos-10.org ]
%
% VERSION NUMBER: 3.06.1 (14th March, 2019)
%
% REFERENCES:
% Klocker et al 2009: A new method of forming approximately neutral 
%  surfaces,., Ocean Science, 5, 155-172.
%
% The software is available from http://www.TEOS-10.org
%
%==========================================================================

Number_of_iterations = 0;
phiprime_flag = 0;
Number_of_iterations_to_go = 10;
rho_SSO_0_0_squared = 1.057004047745510e6;

while phiprime_flag == 0
    
    Number_of_iterations = Number_of_iterations + 1;
    
    if Number_of_iterations == 3 | Number_of_iterations_to_go == 3
        SAwet = squeeze(SA(1,:,:));
        Iwet = find(~isnan(SAwet) & isnan(pns));
        if ~isempty(Iwet)
            wet_loop = 1;
            wet_count = 0;
            while wet_loop == 1
                wet_pnts = sum(~isnan(SAns(:)));
                wet_count = wet_count + 1;
                [SAns,CTns,pns] = gsw_ns2d_wetting_edges(SA,CT,p,SAns,CTns,pns,settings);  
                if wet_pnts == sum(~isnan(SAns(:))) | wet_count == 100
                    wet_loop = 0;
                end
            end
        end
    end
    
    % calculate density gradient errors on the density surface
    [rho_x_error, rho_y_error] = gsw_ns2d_rho_gradient_errors(SAns,CTns,pns,e1t,e2t,settings);
     
    % find dimensions
    [yi,xi] = size(pns);
    
    % preallocate
    phi_prime = nan(yi,xi);
    
    b = zeros(3*xi*yi,1,1);
    s1 = zeros(3*xi*yi,1);
    s2 = zeros(3*xi*yi,1);
    s3 = zeros(3*xi*yi,1);
    
    % find grid points which are not continents
    % find grid points which are ocean
    Iocean = find(~isnan(pns(:)));
    
    % build matrix where the ocean gridpoints have their indices and
    % continet gridpoints nans
    ng = nan(yi,xi);
    ng(Iocean) = Iocean;
        
    % build neighbour matrix -> build a matrix which is like a look-up
    % table to see which gridpoints communicate with each other
    neighbour = gsw_ns2d_neighbour(yi,xi,ng,Iocean,settings);
     
    nregion = 0;
    region_matrix = nan(yi,xi);
    
    % find independent regions -> a least-squares problem is solved for
    % each of these regions
    while (length(find(~isnan(region_matrix))) ~= length(neighbour))
        
        pos = 0;
        region = nan(1,2*length(Iocean));
        nregion = nregion + 1;
        
        % find starting point of region
        [pos,region,region_matrix,neighbour] = gsw_ns2d_region_startingpoint(pos,region,nregion,region_matrix,neighbour,Iocean);
        
        region_old = [];
        
        while(length(region(1:pos)) ~= length(region_old))
            
            region_old = region(1:pos);
            
            % find all other points of region
            [pos,region,region_matrix,neighbour] = gsw_ns2d_region_otherpoints(pos,region,nregion,region_matrix,neighbour,Iocean);
            
        end
        region(pos+1:length(region)) = [];
                
        % build a matrix where all the points of the region are labelled
        % with 1 ,2 , .... , length(region), other regions and continents are
        % filled with NaNs.
        ng_region = nan(yi,xi);
        ng_region(region) = 1:length(region);
        
        neq = 0; % set equation number to 0
        ieq = 0;
        
        % set up east-west equations for weighted inversion
        [neq,ieq,s1,s2,s3,b] = gsw_ns2d_ew_eqns(yi,xi,region,neq,ieq,s1,s2,s3,b,rho_x_error,ng,ng_region,e1t,settings);
        
        % set up north-south equations for weighted inversion
        [neq,ieq,s1,s2,s3,b] = gsw_ns2d_ns_eqns(yi,xi,region,neq,ieq,s1,s2,s3,b,rho_y_error,ng,ng_region,e2t);

        % make the average of all density changes zero -> this should keep
        % the surface from drifting away from the initial condition
        neq = neq + 1;
        
        for i = 1:length(region)
            [jj,ii] = ind2sub([yi,xi],region(i));
            ieq = ieq + 1;
            s1(ieq) = neq;
            s2(ieq) = ng_region(jj,ii);
            s3(ieq) = 1;
        end
        b(neq,1) = 0;
        
        % cut A and b to appropriate size
        s1 = s1(1:ieq);
        s2 = s2(1:ieq);
        s3 = s3(1:ieq);
        b = b(1:neq,1);
        
        % make matrix sparse and invert
        A = sparse(s1,s2,s3);
        b = sparse(b);
        [x, dummy_flag] = lsqr(A,b,1e-7,50000);
        x = full(x)';
       
        % put density changes calculated by the least-squares solver into
        % their appropriate position in the matrix
        phi_prime(region) = x;
        
    end
    
    % calculate properties of the updated neutral surface
    [SAns,CTns,pns] = gsw_ns2d_vertical_solve(SA,CT,p,SAns,CTns,pns,phi_prime);
    
    %calculate the stopping point   
    phiprime = rho_SSO_0_0_squared.*phi_prime(:);
    Idata = find(~isnan(phiprime));
               
    if nansum(abs(phiprime(Idata))/length(Idata)) < 1e-5 & Number_of_iterations_to_go == 10
        Number_of_iterations_to_go = 3;
    elseif Number_of_iterations_to_go <= 3 & Number_of_iterations_to_go > 0
        Number_of_iterations_to_go = Number_of_iterations_to_go - 1;
    elseif Number_of_iterations == 12 | Number_of_iterations_to_go == 0
        phiprime_flag = 1;
    end
    
%     if mod(Number_of_iterations,12) == 1
%         figure('Position',[20, 20, 1000, 800])
%         [hax,hax_noxlabel,hax_noylabel,hax_left,hax_bottom] = nfigaxes([3 4],[0.06 0.06],[0.1 0.9],[0.1 0.9]);
%     end
%     sp_pos = mod(Number_of_iterations,12) ;
%     if sp_pos == 0
%         sp_pos = 12;
%     end
%     
%     axes(hax(sp_pos))
%     pcolor(rho_SSO_0_0_squared.*phi_prime)
%     shading flat
%     if Number_of_iterations_to_go <= 3
%         if Number_of_iterations == 1
%             title(['\phi '' for ',num2str(Number_of_iterations),'^s^t iteration (w)'],'fontsize',20,'fontweight','bold')
%         elseif Number_of_iterations == 2
%             title(['\phi '' for ',num2str(Number_of_iterations),'^n^d iteration (w)'],'fontsize',20,'fontweight','bold')
%         elseif Number_of_iterations == 3
%             title(['\phi '' for ',num2str(Number_of_iterations),'^r^d iteration (w)'],'fontsize',20,'fontweight','bold')
%         else
%             title(['\phi '' for ',num2str(Number_of_iterations),'^t^h iteration (w)'],'fontsize',20,'fontweight','bold')
%         end
%     else
%         if Number_of_iterations == 1
%             title(['\phi '' for ',num2str(Number_of_iterations),'^s^t iteration '],'fontsize',20,'fontweight','bold')
%         elseif Number_of_iterations == 2
%             title(['\phi '' for ',num2str(Number_of_iterations),'^n^d iteration '],'fontsize',20,'fontweight','bold')
%         elseif Number_of_iterations == 3
%             title(['\phi '' for ',num2str(Number_of_iterations),'^r^d iteration '],'fontsize',20,'fontweight','bold')
%         else
%             title(['\phi '' for ',num2str(Number_of_iterations),'^t^h iteration '],'fontsize',20,'fontweight','bold')
%         end
%     end
%     caxis([(mean(phiprime(Idata)) - 2*std(phiprime(Idata))) (mean(phiprime(Idata)) + 2*std(phiprime(Idata)))])
%     colorbar
%     
%     pause(0.1)
        
end

end



