function [rho_gradient_error_x,rho_gradient_error_y] = gsw_ns2d_rho_gradient_errors(SAns,CTns,pns,e1t,e2t,settings)

% gsw_ns2d_rho_gradient_errors                      density gradient errors
%==========================================================================
% 
% USAGE:  
%  [rho_gradient_error_x,rho_gradient_error_y] = ...
%              gsw_ns2d_rho_gradient_errors(SAns,CTns,pns,e1t,e2t,settings)
%
% DESCRIPTION:
%  Calculate the density gradient errors in the East-West and North-South
%  directions.
%
% Input:    
%  SAns = Absolute Salinity on neutral surface
%  CTns = Conservative Temperature on neutral surface
%  pns =  pressure on neutral surface
%  e1t =  zonal scale-factor at tracer points
%  e2t =  meridional scale-factor at tracer points
%
% Output:   
%  rho_gradient_error_x = x-component of density gradient errors
%  rho_gradient_error_x = y-component of density gradient errors
%
% AUTHOR: Andreas Klocker
%  Modified by Paul Barker
%
% VERSION NUMBER: 3.06.1 (14th March, 2019)
%
% REFERENCES:
% Klocker et al 2009: A new method of forming approximately neutral 
%  surfaces,., Ocean Science, 5, 155-172.
%
%==========================================================================

% check input arguments
if ~(nargin == 6)
    error('gsw_ns2d_rho_gradient_errors: requires 6 input arguments')
end

[SAns_x, SAns_y, CTns_x, CTns_y] = gsw_ns2d_SA_CT_xy_gradients(SAns,CTns,e1t,e2t,settings);

[meanSAns_x,meanSAns_y, meanCTns_x,meanCTns_y, meanpns_x,meanpns_y] = gsw_ns2d_SA_CT_p_xy_mean(SAns,CTns,pns,settings);

[v_SA_x, v_CT_x, dummy] = gsw_specvol_first_derivatives(meanSAns_x,meanCTns_x,meanpns_x);
[v_SA_y, v_CT_y, dummy] = gsw_specvol_first_derivatives(meanSAns_y,meanCTns_y,meanpns_y);

rho_gradient_error_x = (-v_SA_x .* SAns_x) - (v_CT_x .* CTns_x);
rho_gradient_error_y = (-v_SA_y .* SAns_y) - (v_CT_y .* CTns_y);

end