function [e2t,e1t] = gsw_ns2d_scaling_factor(lats,longs,settings)

% gsw_ns2d_scaling_factor                                    scaling factor
%==========================================================================
% 
% Usage:  
%  [e2t,e1t] = gsw_ns2d_scaling_factor(lats,longs,settings)
%
% DESCRIPTION:
%  Find distances between gridpoints of given latitude/longitude in a
%  2-dimension dataset.
%
% Input: 
%  lats  = latitude
%  longs = longitude
%  settings.wrap = 'none' or 'long'
%
% Output:   
%  e2t  =  distance between gridpoints in N-S direction               [ m ]
%  e1t  =  distance between gridpoints in E-W direction               [ m ]
%
% AUTHOR: Andreas Klocker
%  Modified by Paul Barker
%
% VERSION NUMBER: 3.06.1 (14th March, 2019)
%
% REFERENCES: 
% Klocker et al 2009: A new method of forming approximately neutral 
%  surfaces,., Ocean Science, 5, 155-172.
%
%==========================================================================

% check input arguments
if ~(nargin == 3)
    error('scale_fac: requires 3 input arguments')
end

% initialize and preallocate memory
[yi,xi] = size(lats);
e1t = nan(yi,xi);
e2t = nan(yi,xi);

% calculate distances
switch settings.wrap
    
    case 'none'
        for j = 1:yi
            for i = 1:xi-1
                e1t(j,i) = gsw_distance([longs(j,i) longs(j,i+1)],[lats(j,i) lats(j,i)]);
            end
        end
        for j = 1:yi-1
            for i = 1:xi
                e2t(j,i) = gsw_distance([longs(j,i) longs(j,i)],[lats(j,i) lats(j+1,i)]);
            end
        end
        
    case 'long'
        for j = 1:yi
            for i = 1:xi-1
                e1t(j,i) = gsw_distance([longs(j,i) longs(j,i+1)],[lats(j,i) lats(j,i)]);
            end
        end
        for j = 1:yi
            e1t(j,xi) = gsw_distance([longs(j,xi) longs(j,1)],[lats(j,i) lats(j,i)]);
        end
        for j = 1:yi-1
            for i = 1:xi
                e2t(j,i) = gsw_distance([longs(j,i) longs(j,i)],[lats(j,i) lats(j+1,i)]);
            end
        end
        
end

end
