function [SAns, CTns, pns] = gsw_ns2d_wetting_edges(SA,CT,p,SAns,CTns,pns,settings)

% gsw_ns2d_wetting_edges        Absolute Salinity, Conservative Temperature
%                                 and pressure on a neighbouring cast using
%                             the neutral tangent plane  (75-term equation)
%==========================================================================
%
% USAGE:
%  [SAns, CTns, pns] = ...
%                    gsw_ns2d_wetting_edges(SA,CT,p,SAns,CTns,pns,settings)
%
% DESCRIPTION:
%  This function is for use on gridded data, it finds an estimate of the 
%  Absolute Salinity, Conservative Temperature and pressure on casts that
%  do not have an estimate of the values on a neutral surface. It 
%  extrapolates the neutral suface to neighbourning casts, i.e. it wets the
%  edges of neutral surfaces.
%
%  Note that this function is intended to work on stabile profiles.  If 
%  there are instabillities then there are mupliple solutions it will
%  generally not find all the crossings, the results may including stable 
%  unstable zero crossings.
%
% INPUT:  
%  SA  =  Absolute Salinity of the cast                            [ g/kg ]
%  CT  =  Conservative Temperature of the cast                    [ deg C ]
%  p   =  sea pressure of the  cast                                [ dbar ]
%         ( i.e. absolute pressure - 10.1325 dbar )
%  SAns = Absolute Salinity of the neutral surface                 [ g/kg ]
%  CTns = Conservative Temperature of the neutral surface(ITS-90  [ deg C ]
%  pns  = sea pressure of the neutral surface                      [ dbar ]
%         ( i.e. absolute pressure - 10.1325 dbar )
%  settings = settings from gsw_ns2d
%
%  SA, CT and p need to have the same dimensions MxNxO, where M is the
%  number of bottles in the cast and N is the number of profiles in the 
%  east-west direction and O is the number of profiles in the north-south 
%  direction.  Note that they need to be arranged in a columns, if they are
%  inputed as a row the programme will assume they are seperate casts. 
%  SAns, CTns and pns need to be values on a single surface, having 
%  dimensions NxO.
%  settings needs to be a structure.
%
% OUTPUT:
%  SAns = Absolute Salinity of the neutral surface                 [ g/kg ]
%  CTns = Conservative Temperature of the neutral surface         [ deg C ]
%  pns  = pressure of the neutral surface                          [ dbar ]
%
% AUTHOR: Stefan Riha
%  Modified by Paul Barker
%
% VERSION NUMBER: 3.06.1 (14th March, 2019)
%
% REFERENCES: 
% Klocker et al 2009: A new method of forming approximately neutral 
%  surfaces,., Ocean Science, 5, 155-172.
%
%  The software is available from http://www.TEOS-10.org
%
%==========================================================================

[yi,xi] = size(SAns);

wet = ~isnan(squeeze(SA(1,:,:))) & isnan(SAns); % wet points at ocean surface excluding the neutral surface
wets = ~isnan(SAns);                            % wet points on the neutral surface

north = wet(:) & circshift(wets(:),-1);         % wet points with north neighbour on the neutral surface
south = wet(:) & circshift(wets(:),1);
east = wet(:) & circshift(wets(:),-yi);
west = wet(:) & circshift(wets(:),yi);

north(yi:yi:yi*xi) = false;
south(1:yi:(xi-1)*yi+1) = false;

if ~strncmp(settings.wrap,'long',4)
    east((xi-1)*yi+1:xi*yi) = false;
    west(1:yi) = false;
end

% if a point adjacent to the neutral surface boundary has multiple neighbours we will only do one neutral calculation
west = west & ~east;
north = north & ~west & ~east;
south = south & ~north & ~west & ~east;

I = [1:xi*yi];
inds_neighbour = circshift(I,-yi);
neighbour = inds_neighbour(east);

[SAns(east),CTns(east),pns(east)] = gsw_ntp_bottle_to_cast(SAns(neighbour),CTns(neighbour),pns(neighbour),SA(:,east),CT(:,east),p(:,east));

inds_neighbour = circshift(I,yi);
neighbour = inds_neighbour(west);
[SAns(west),CTns(west),pns(west)] = gsw_ntp_bottle_to_cast(SAns(neighbour),CTns(neighbour),pns(neighbour),SA(:,west),CT(:,west),p(:,west));

inds_neighbour = circshift(I,-1);
neighbour = inds_neighbour(north);
[SAns(north),CTns(north),pns(north)] = gsw_ntp_bottle_to_cast(SAns(neighbour),CTns(neighbour),pns(neighbour),SA(:,north),CT(:,north),p(:,north));

inds_neighbour = circshift(I,1);
neighbour = inds_neighbour(south);
[SAns(south),CTns(south),pns(south)] = gsw_ntp_bottle_to_cast(SAns(neighbour),CTns(neighbour),pns(neighbour),SA(:,south),CT(:,south),p(:,south));

sum_e = sum(~isnan(SAns(east)));
sum_w = sum(~isnan(SAns(west)));
sum_n = sum(~isnan(SAns(north)));
sum_s = sum(~isnan(SAns(south)));

sum_neighbours = sum_e + sum_w + sum_n + sum_s;

Iwetted = ~isnan(SAns(:)) & (east | west | south | north);

if sum(Iwetted) ~= sum_neighbours
    error('gsw_ns2d_wetting_edges')
end

end



